﻿(* Raima.Rdm is the system module that contains the ADO.NET data provider
   for RDM *)
open Raima.Rdm

(* EXAMPLE - SQL10 for F# using ADO.NET *)
(* This example shows multi-user access to a database.  Two connections are
 * established using shared mode.  In one data is inserted into the database.
 * The other one also opens the database.  The second connection is used to
 * read the data while the first connection still has a transaction open,
 * which fails with a timeout.  Then the transaction is committed (freeing
 * the locks) and the read using the second connection then succeeds.
 * 
 * The following methods are illustrated:
 *     Raima.Rdm.RdmConnection:
 *         BeginTransaction() Method
 *         ChangeDatabase(string) Method
 *         Close() Method
 *         CreateCommand() Method
 *         Open() Method
 *         RdmConnection(String) Constructor
 *     Raima.Rdm.RdmCommand:
 *         CommandText Property
 *         ExecuteNonQuery() Method
 *         ExecuteReader() Method
 *     Raima.Rdm.RdmDataReader:
 *         Item[int] Property
 *         Read() Method
 *     Raima.Rdm.RdmTransaction
 *         Commit() Method
 * 
 * They are fully document in http://docs.raima.com/rdm/.
 * 
 * For simplicity this example does not catch any thrown exceptions
 * but good programming practices would dictate that this occur in
 * a full application.
 *)

let DropDatabase (conn : RdmConnection) name =
    let cmd = conn.CreateCommand()

    (* Attempt to drop this database.  If the database does not already exist
     * this will fail and we simply ignore the failure for this sample. *)
    try
        cmd.CommandText <- "DROP DATABASE " + name
        cmd.ExecuteNonQuery() |> ignore
    with
        | exn -> ()

let CreateThisDatabase (conn : RdmConnection) name cmds =
    let cmd = conn.CreateCommand()

    cmd.CommandText <- "CREATE DATABASE " + name
    cmd.ExecuteNonQuery() |> ignore

    cmds |>
        Array.iter(fun c ->
            cmd.CommandText <- c
            cmd.ExecuteNonQuery() |> ignore
        )

    cmd.CommandText <- "COMMIT"
    cmd.ExecuteNonQuery() |> ignore

    cmd.CommandText <- "CLOSE " + name
    cmd.ExecuteNonQuery() |> ignore

let CreateDatabases conn =
    (* Drop any existing copies of the databases. *)
    DropDatabase conn "SQL10"

    (* Attempt to create the databases *)
    CreateThisDatabase conn "SQL10" [| "CREATE TABLE Table1 (id INTEGER PRIMARY KEY)" |]

(* Create the connection with a simple connection string *)
let conn1 = new RdmConnection("host=local;openmode=s")
conn1.Open()

CreateDatabases conn1

(* Open the database *)
conn1.ChangeDatabase("SQL10")

(* Create a second connection *)
let conn2 = new RdmConnection("host=local;database=SQL10;openmode=s;timeout=2")
conn2.Open()

(* Create a command in connection 1 and one in connection 2 *)
let cmd1 = conn1.CreateCommand()
let cmd2 = conn2.CreateCommand()

(* Start a new transaction in connection 1 *)
let trans = conn1.BeginTransaction()

(* Insert three rows into the table using connection 1 *)
[1..3] |>
    List.iter(fun ii ->
        cmd1.CommandText <- "INSERT INTO Table1 VALUES " + (string ii)
        cmd1.ExecuteNonQuery() |> ignore
    )

(* Attempt to read the data from connection 2; expect a timeout
             * because connection 1 is holding locks as the transaction is 
             * still active *)
try
    cmd2.CommandText <- "SELECT id FROM Table1"
    let rdr1 = cmd2.ExecuteReader()

    while (rdr1.Read()) do
        printfn "%s" (string (rdr1.Item(0)))
with
    | :? RdmException as exn ->
        match exn.Message with
        | "timeout expired" -> printfn "Received expected timeout"
        | _ -> printfn "Unexpected exception %A" exn

(* Commit the transaction *)
trans.Commit()

(* Attempt to read the data again, which should now succeed *)
cmd2.CommandText <- "SELECT id FROM Table1"
let rdr2 = cmd2.ExecuteReader()

while (rdr2.Read()) do
    printfn "%s" (string (rdr2.Item(0)))

conn1.Close()
conn2.Close()

printfn "Hit <Enter> to continue"
System.Console.ReadLine() |> ignore

